from graphene import relay
from social_django import models as social_models

from .. import types


class SocialNode(types.SocialType):

    class Meta:
        model = social_models.UserSocialAuth
        interfaces = [relay.Node]
        filter_fields = {
            'uid': ['exact', 'in'],
            'provider': ['exact', 'in'],
        }



from graphene.types.generic import GenericScalar
from graphene_django.types import DjangoObjectType
from social_django import models as social_models

from .utils import dashed_to_camel


class CamelJSON(GenericScalar):

    @classmethod
    def serialize(cls, value):
        return dashed_to_camel(value)

    class Meta:
        name = 'SocialCamelJSON'


class SocialType(DjangoObjectType):
    extra_data = CamelJSON()

    class Meta:
        model = social_models.UserSocialAuth

    def resolve_extra_data(self, info, **kwargs):
        self.extra_data.pop('access_token', None)
        return self.extra_data



import graphene

from . import mixins, types
from .decorators import social_auth


class SocialAuthMutation(mixins.SocialAuthMixin, graphene.Mutation):
    social = graphene.Field(types.SocialType)

    class Meta:
        abstract = True

    class Arguments:
        provider = graphene.String(required=True)
        access_token = graphene.String(required=True)

    @classmethod
    @social_auth
    def mutate(cls, root, info, social, **kwargs):
        return cls.do_auth(info, social, **kwargs)


class SocialAuth(mixins.DoAuthMixin, SocialAuthMutation):
    """Social Auth Mutation"""


class SocialAuthJWT(mixins.DoAuthJWTMixin, SocialAuthMutation):
    """Social Auth for JWT (JSON Web Token)"""



import graphene


class SocialAuthMixin(object):

    @classmethod
    def __init_subclass_with_meta__(cls, name=None, **options):
        assert getattr(cls, 'do_auth', None), (
            '{name}.do_auth method is required in a SocialAuthMutation.'
        ).format(name=name or cls.__name__)

        super().__init_subclass_with_meta__(name=name, **options)


class DoAuthMixin(object):

    @classmethod
    def do_auth(cls, info, social, **kwargs):
        return cls()


class DoAuthJWTMixin(object):
    token = graphene.String()

    @classmethod
    def do_auth(cls, info, social, **kwargs):
        try:
            from graphql_jwt.shortcuts import get_token
        except ImportError:
            raise ImportError(
                'django-graphql-jwt not installed.\n'
                'Use `pip install \'django-graphql-social[jwt]\'`.'
            )
        return cls(token=get_token(social.user))



from functools import wraps

from django.contrib.auth import login
from django.utils.translation import ugettext_lazy as _

from promise import Promise, is_thenable
from social_core.exceptions import MissingBackend
from social_django.utils import load_backend, load_strategy

from . import exceptions


def psa(f):
    @wraps(f)
    def wrapper(cls, root, info, provider, access_token, **kwargs):
        strategy = load_strategy(info.context)
        try:
            backend = load_backend(strategy, provider, redirect_uri=None)
        except MissingBackend:
            raise exceptions.GraphQLSocialError(_('Provider not found'))

        user = backend.do_auth(access_token)

        if user is None:
            raise exceptions.GraphQLSocialError(_('Invalid token'))

        login(info.context, user)
        social = user.social_auth.get(provider=provider)

        return f(cls, root, info, social, **kwargs)
    return wrapper


def social_auth(f):
    @psa
    @wraps(f)
    def wrapper(cls, root, info, social, **kwargs):
        def on_resolve(payload):
            payload.social = social
            return payload

        result = f(cls, root, info, social, **kwargs)

        # Improved mutation with thenable check
        if is_thenable(result):
            return Promise.resolve(result).then(on_resolve)
        return on_resolve(result)
    return wrapper



import graphene

from . import nodes
from .. import mixins, mutations
from ..decorators import social_auth


class SocialAuthMutation(mixins.SocialAuthMixin,
                         graphene.relay.ClientIDMutation):

    social = graphene.Field(nodes.SocialNode)

    class Meta:
        abstract = True

    class Input(mutations.SocialAuthMutation.Arguments):
        """Social Auth Input"""

    @classmethod
    @social_auth
    def mutate_and_get_payload(cls, root, info, social, **kwargs):
        return cls.do_auth(info, social, **kwargs)


class SocialAuth(mixins.DoAuthMixin, SocialAuthMutation):
    """Social Auth Mutation for Relay"""


class SocialAuthJWT(mixins.DoAuthJWTMixin, SocialAuthMutation):
    """Social Auth for JWT (JSON Web Token)"""
